#	include	"Standard.h"
#	include	"mygl.h"
#	include	"render.h"

// sanity check only support files of < this number of vertexs/polygons
#	define	MAX_FILE_SIZE	20000

// This module reads a .OFF (see: http://www.geomview.org/docs/oogltour.html)
// file format.

// Read a .OFF file and return a list of polygons (all white)
// Very limited error checking.... and only polygon .OFF file support

Array<Polygon>	*
ReadOffFile(
	char	*filename)
	{
	FILE	*fp = fopen(filename, "r");
	if ( ! fp )
		return ( NULL );
	char	s[2048];

	fgets(s, 2000, fp);

	if ( strncmp(s, "OFF", 3) )
		{
		printf("Not an OFF file?\n");
		fclose(fp);
		return(NULL);
		}

	int	NumberOfVertexs;
	int	NumberOfPolygons;
	int	NumberOfEdges;

	fscanf(fp, "%d %d %d", &NumberOfVertexs, &NumberOfPolygons, &NumberOfEdges);

	if ( NumberOfVertexs <= 0 || NumberOfVertexs > MAX_FILE_SIZE )
		{
		printf("NumberOfVertexs = %d is out of range\n", NumberOfVertexs);
		fclose(fp);
		return(NULL);
		}

	if ( NumberOfPolygons <= 0 || NumberOfPolygons > MAX_FILE_SIZE )
		{
		printf("NumberOfPolygons = %d is out of range\n", NumberOfPolygons);
		fclose(fp);
		return(NULL);
		}


	Vertex	*VertexList = new Vertex[NumberOfVertexs];

	int		Index = 0;
	while ( Index < NumberOfVertexs )
		{
		fscanf(fp, "%f %f %f", &VertexList[Index].x, &VertexList[Index].y, &VertexList[Index].z);
		VertexList[Index].color = White;
		++Index;
		}

	int	EdgeCount;
	int	v1, v2, v3;
	Array<Polygon>	*PolygonList = new Array<Polygon>;
	Polygon	p;

	Index = 0;
	while ( Index < NumberOfPolygons )
		{
		fscanf(fp, "%d %d %d %d", &EdgeCount, &v1, &v2, &v3);
		if ( EdgeCount != 3 )
			{
			printf("Sorry, currently I'm too stupid to understand anything but polygons\n");
			fclose(fp);
			delete PolygonList;
			delete VertexList;
			return(NULL);
			}
		if ( (v1 < 0 || v1 >= NumberOfVertexs) ||
			 (v2 < 0 || v2 >= NumberOfVertexs) ||
			 (v3 < 0 || v3 >= NumberOfVertexs) )
			{
			printf("Corrupt file, vertex index is out of range at polygon %d\n", Index);
			fclose(fp);
			delete PolygonList;
			delete VertexList;
			return(NULL);
			}

		p.v1 = VertexList[v1];
		p.v2 = VertexList[v2];
		p.v3 = VertexList[v3];
		PolygonList->Add(p);
		++Index;
		}
	fclose(fp);
	delete VertexList;

	// cool
	return(PolygonList);
	}

